#!/usr/bin/perl -w
#
# this script corrects an .m3u file, all of the mp3's it references, 
# as well as the base dir it's in, based on a corrected version of the .m3u file

# first argument is the original .m3u file you want to correct
# second argument is the new one; this can be left out, in which case
# it will default to the first one with .new appended
#
# don't change the ordering of tracks either, just correct them

use strict;
use utf8;
no bytes;
use Encode;

use vars qw($LOGFUNC $BASE $PROGRAM);

$PROGRAM = "dad-albumcorrect";
my $version = "0.2.5";

my $conf = "/etc/dad.conf";
require $conf;
require $LOGFUNC;

# how to tag files with artist and title
my %tag =
(
  "mp1" => "id3 -a \"%a\" -t \"%t\" \"%f\" > /dev/null 2>&1",
  "mp2" => "id3 -a \"%a\" -t \"%t\" \"%f\" > /dev/null 2>&1",
  "mp3" => "id3 -a \"%a\" -t \"%t\" \"%f\" > /dev/null 2>&1",
);

use File::Basename;
# Getopt stuff
use Getopt::Std;
my %opt;

sub check_arguments ($)
# check arguments and fill in all relevant data in the opt_ref
{
  my $opt_ref = shift;
  
  if (!getopts ('hd', $opt_ref))
  {
    usage ();
    end_program (1);
  }
}                 

sub parse_filename ($)
# parse the filename in path, artists, title, extension
{
  my ($path, $artists, $title, $extension);
  my $inputname = shift;
  
  # FIXME : add more filename checking here
  $inputname =~ /^(.+)\s+-\s+(.+)\.([^\.]+)$/;
  if (!$1 || !$2 || !$3)
  {
    logger ("E3", "could not parse file name into components");
    end_program (1);
  }
  
  my $pathartists = $1;
  $title = $2;
  $extension = $3;

  if ($pathartists =~ /\//)
  {
    # it contains a path separator, so extract
    $pathartists =~ /^(.+)\/(.+)/;
    $path = $1;
    $artists = $2;
  }
  else
  {
    # no path, so it's the current dir
    # FIXME: maybe we should put the current path here ?
    $path = get_pwd ();
    $artists = $pathartists;
  }
      
  return ($path, $artists, $title, $extension);
}

sub get_artists ($)
# check if the given artistlist string contains more than one artist
# (separated with &) and return a list of separate artists
{
  my $artists = shift;
  my $artist;

  my @artistlist = split " & ", $artists;

  foreach $artist (@artistlist)
  {
    #print "DEBUG: $artist\n";
  }
  return @artistlist;
}

sub get_pwd ()
# return the current directory
{
 my $pwd = `pwd`;
 chomp $pwd;
 return $pwd;
}


sub usage ()
# print usage information
{
  print <<END;
$PROGRAM $version, (C) thomas\@apestaart.org

$PROGRAM [old .m3u] ([new .m3u])

Full path (starting with \/) must be specified.
If the new .m3u file is not specified, old.m3u.new is assumed.

END
}

sub end_program ($)
# log exit and exit
{
  my $value = shift;
  logger ("S3", "stopped.");
  exit $value;
}

# main program

logger ("S3", "started.");

# check arguments

check_arguments (\%opt);

my $oldm3u = decode_utf8 (shift @ARGV) or die "Please give the original .m3u file";
my $newm3u = decode_utf8 (shift @ARGV) || "$oldm3u.new";

print "DEBUG: oldm3u: $oldm3u\n";
print "DEBUG: newm3u: $newm3u\n";

if ($oldm3u !~ /^\//)
{
  $oldm3u = get_pwd()."/$oldm3u";
}
if ($newm3u !~ /^\//)
{
  $newm3u = get_pwd()."/$newm3u";
}
if (! -e $oldm3u)
{
  die "Old m3u file $oldm3u does not exist !";
}

if (! -e $newm3u)
{
  die "New m3u file $newm3u does not exist !";
}

# read in old and new m3u's
open OLD, $oldm3u or die "Cannot open $oldm3u for reading";
my @old = <OLD>;
close OLD;
open NEW, $newm3u or die "Cannot open $newm3u for reading";
my @new = <NEW>;
close NEW;

my ($base, $dir, $ext) = fileparse ($oldm3u);

# rename tracks and retag them with artist and title

my $i;
for ($i = 0; $i < scalar (@old); ++$i)
{
  my $old = $old[$i];
  my $new = $new[$i];
  chomp ($old);
  chomp ($new);
  
  if ($old ne $new)
  {
    logger ("D4", "renaming $old to $new");
    if ($opt{"d"})
    {
      print "mv \"$dir/$old\" \"$dir/$new\"\n";
    }
    else
    {
      if (!rename ("$dir/$old", "$dir/$new"))
      {
        logger ("E3", "could not rename $dir/$old to $dir/$new !");
      }
    }
    my ($path, $artists, $title, $ext) = parse_filename ($new);
    logger ("D4", "tagging $new with artists $artists and title $title");
    # give ogg's special treatment because they're so damn hard to do
    # list the tags, grep out ARTIST and TITLE, and put the good ones in
    if ($ext eq "ogg")
    {
      my $tag = `vorbiscomment -l \"$dir/$new\" | grep -v \"ARTIST=\" | grep -v \"TITLE=\"`;
      $tag .= "ARTIST=$artists\nTITLE=$title\n";
      open TAG, "| vorbiscomment -w \"$dir/$new\"";
      
      print TAG $tag;
      close TAG;
    }
    elsif (my $command = $tag{$ext})
    {
      # s & r the % parameters by their values
      $command =~ s@%f@$dir/$new@;
      $command =~ s/%a/$artists/;
      $command =~ s/%t/$title/;
      logger ("D4", "$command");
      system ($command);
    }
  }
}

# remove old m3u
if ($opt{"d"}) { print "rm \"$oldm3u\"\n"; } else { unlink $oldm3u; }

# replace with new m3u
$newm3u =~ /^(.+)\.new$/;
my $m3u = $1;
if ($opt{"d"})
{
  print "mv \"$newm3u\" \"$m3u\"\n";
}
else
{
  rename ($newm3u, $1) or system ("mv", $newm3u, $1)
}
# rename dir
# get old base by popping off the m3u file from the oldm3u path
# pop off the dir name (= old album name) to get the base for new dir
# get new dir by adding the $newm3u name, minus m3u stuff, to this base

#print "DEBUG: oldm3u $oldm3u\n";
my $olddir = dirname ($oldm3u);
$base = dirname ($olddir);
my $newdir = $base."/".basename ($newm3u);
$newdir =~ s/\.m3u.*$//;

#print "DEBUG: olddir $olddir, newdir $newdir\n";

if ($olddir ne $newdir)
{
  logger ("I4", "renaming $olddir to $newdir");
  if ($opt{"d"})
  {
    print "cd ..\nmv \"$olddir\" \"$newdir\"\n";
  }
  else
  {
    chdir "..";
    rename ($olddir, $newdir) or system ("mv", $olddir, $newdir)
  }
}
# finish
end_program (0);
