#!/usr/bin/perl -w
#
# this script imports albums into the dad system
# it scans the given .m3u file, which should be an album playlist
# each of the audio files should already be in the database
# all of the tracks get their right track number assigned

use strict;

use vars qw($LOGFUNC $BASE $PROGRAM
            $DB_HOST $DB_USER $DB_PASS $DB_NAME $DB_DRIVER
            $DAD_USER_ID $SONG_TYPE_ID $AUDIO_DIR);
use utf8;
use Encode;
            

$PROGRAM = "dad-albumimport";
my $version = "0.2.5";

##$ENV{DAD_BASEDIR} or die "DAD_BASEDIR environment variable not set !";

my $conf = "/etc/dad.conf";
require $conf;
require $LOGFUNC;

# Getopt stuff
use Getopt::Std;
my %opt;

# database stuff
use DBI;

# subroutines

sub check_arguments ($)
# check arguments and fill in all relevant data in the opt_ref 
{
  my $opt_ref = shift;

  if (!getopts ('h', $opt_ref))
  {
    usage ();
    end_program (0);
  }
  if (!($opt_ref->{"f"} = decode_utf8 ($ARGV[0])))
  {
    # no filename given
    logger ("E3", "no filename given !");
    end_program ();
  }
  if (! -e $ARGV[0])
  {
    # filename does not exist
    logger ("E3", "file not found !");
    end_program ();
  }
  
  logger ("I3", "importing album file ".$opt_ref->{"f"});
}

sub connect_database ()
# connect to database, exit if fail
# return the database handler
{
  logger ("D3", "connecting to database");
  my $data_source = "DBI:$DB_DRIVER:$DB_NAME:$DB_HOST";
  # FIXME: not sure about this AutoCommit and relationship with rollback
  my $dbh = DBI->connect ($data_source, $DB_USER, $DB_PASS,
                          { AutoCommit => 1 } );
  if (!$dbh)
  {
    logger ("E3", "could not connect to database");
    end_program ();
  }
  return $dbh;
}

sub db_get_audiofile_id ($$)
# look up the audio filename in the database
# return the audiofile_id
{
  my $dbh = shift;
  my $filename = shift;

  chomp ($filename);

  my ($path, $artists, $title, $extension) = parse_filename ($filename);
  my $basefilename = "$artists - $title.$extension";

  my $quotedfilename = $dbh->quote ($basefilename);
  my $quotedpath = $dbh->quote ($path);

  # print "DEBUG: looking for filename $quotedfilename in path $path\n";
  # first check if it's in the database already
  my $query = "SELECT audiofile_id FROM audiofile JOIN paths WHERE filename=$quotedfilename AND path=$quotedpath";
  my $sth = $dbh->prepare ($query);
  $sth->execute ();
  if ($sth->rows == 0)
  {
    logger ("E3", "No audiofile with filename=$quotedfilename and path=$quotedpath in database"); 
    $sth->finish();
    end_program ($dbh);
  }
  my $audiofile_id = $sth->fetchrow_array ();
  return $audiofile_id;
}

sub db_get_audio_id ($$)
# get the audio id from the audiofile id supplied
{
  my $dbh = shift;
  my $audiofile_id = shift;
  
  my $query = "SELECT audio_id FROM audio WHERE audiofile_id=$audiofile_id";
  my $sth = $dbh->prepare ($query);
  $sth->execute ();
  if ($sth->rows != 1)
  {
    logger ("E3", "database : ".$sth->rows." audio items found for audiofile_id $audiofile_id");
    end_program ($dbh);
  }
  my $audio_id = $sth->fetchrow_array;
  return $audio_id;
}

sub db_get_artist_id_list ($@)
# get an artist id for each of the mentioned artists, inserting if necessary
{
  my $dbh = shift;
  my @artist_list = @_;

  my @artist_id_list;
  my $artist;

  foreach $artist (@artist_list)
  {
    my $quotedartist = $dbh->quote ($artist);
    # query for this artist
    my $query = "SELECT artist_id FROM artist WHERE artistname=$quotedartist";
    my $sth = $dbh->prepare ($query);
    $sth->execute ();
    if ($sth->rows == 0)
    {
      # it's not in there, we need to add it first !
      my $first_letter_id = db_get_first_letter_id ($dbh, $artist);
      #print "DEBUG: first letter id of $artist : $first_letter_id\n";
      my $query = "INSERT INTO artist VALUES ('',$quotedartist,'$first_letter_id')";
      $sth = $dbh->prepare ($query);
      $sth->execute ();
      my $artist_id = $dbh->{'mysql_insertid'}; # the id we just inserted
      logger ("I4", "added $artist to table artist with id $artist_id");
      push @artist_id_list, $artist_id;
    }
    else
    {
      if ($sth->rows != 1)
      {
        # it's more than 1
        logger ("E3", "database problem : $artist ".$sth->rows." times in table");
        end_program ($dbh);
      }
      my $artist_id = $sth->fetchrow_array ();
      push @artist_id_list, $artist_id;
    }
  }
  #print "DEBUG: (@artist_id_list)\n";
  return @artist_id_list;
}

sub db_get_first_letter_id ($$)
# get id of first letter of artist; create if not there
{
  my $dbh = shift;
  my $artist = shift;
  
  my $first_letter_id;
  
  my $letter = substr ($artist, 0, 1);
  #print "DEBUG: db_get_first_letter_id: letter $letter\n";
  my $query = "SELECT first_letter_id FROM firstletter WHERE letter='$letter'";
  my $sth = $dbh->prepare ($query);
  $sth->execute ();
  #print "DEBUG: db_get_first_letter_id: num of rows ".$sth->rows."\n";
  if ($sth->rows == 0)
  {
    # it's not in there, we need to add it first !
    my $query = "INSERT INTO firstletter VALUES ('','$letter')";
    $sth = $dbh->prepare ($query);
    $sth->execute ();
    $first_letter_id = $dbh->{'mysql_insertid'}; # the id we just inserted
    logger ("I4", "added $letter to table firstletter with id $first_letter_id");
    return $first_letter_id;
  }
  else
  {
    if ($sth->rows != 1)
    {
      # it's more than 1
      logger ("E3", "database problem : $letter ".$sth->rows." times in table");
      end_program ($dbh);
    }
    $first_letter_id = $sth->fetchrow_array ();
  }
  return $first_letter_id;
}

sub db_get_album_id ($$@)
# get id of album; create if not there
# album is combination of album title and list of artists
{
  my $dbh = shift;
  my $album = shift;
  my @artist_id_list = shift;
  
  my $album_id;
  my @album_id_list = ();
  my $artist_id_list = join " ", @artist_id_list;

  my $quotedalbum = $dbh->quote ($album);
  my $query = "SELECT album_id FROM album WHERE title=$quotedalbum";
  my $sth = $dbh->prepare ($query);
  $sth->execute ();
  if ($sth->rows > 0)
  {
    # title exists already, so check the artist-album crosstable
    while ($album_id = $sth->fetchrow_array ()) 
    { push @album_id_list, $album_id; }

    foreach $album_id (@album_id_list)
    {
      my @artist_id_list_db = db_album_get_artist_id_list ($dbh, $album_id);
      my $artist_id_list_db = join " ", @artist_id_list_db;
      if ($artist_id_list_db == $artist_id_list)
      {
        ## print "DEBUG: album already in database with id $album_id !\n";
        logger ("E3", "database problem : $album already in database with id $album_id !\n");
        end_program ($dbh);
      }
    }
  }
  # it's not in yet, so add it
  $query = "INSERT INTO album VALUES ('',$quotedalbum)";
  $sth = $dbh->prepare ($query);
  $sth->execute ();
  $album_id = $dbh->{'mysql_insertid'}; # the id we just inserted
  logger ("I4", "added $album to table album with id $album_id");
  return $album_id;
}

sub db_album_get_artist_id_list ($$)
{
  my $dbh = shift;
  my $album_id = shift;

  my @artist_id_list = ();
  my $artist_id;
  my $query = "SELECT artist_id FROM albumxartist WHERE album_id=$album_id";
  my $sth = $dbh->prepare ($query);
  $sth->execute ();
  if ($sth->rows == 0)
  {
    return ();
  }
  # construct the list
  while ($artist_id = $sth->fetchrow_array ()) 
  { push @artist_id_list, $artist_id; }
  ## print "DEBUG: artist_id_list (@artist_id_list)\n";

  return sort @artist_id_list;
}
 
sub db_audio_add_to_album ($$$$)
# add the album info to the audio item
{
  my $dbh = shift;
  my $audio_id = shift;
  my $album_id = shift;
  my $tracknum = shift;

  my $query = "UPDATE audio SET album_id=$album_id,track=$tracknum WHERE audio_id=$audio_id";
  my $sth = $dbh->prepare ($query);
  $sth->execute ();
  return;
}
sub db_album_artist_cross ($$@)
{
  my $dbh = shift;
  my $album_id = shift;
  my @artist_id_list = @_;

  my $artist_id;

  foreach $artist_id (@artist_id_list)
  {
    my $query = "INSERT INTO albumxartist VALUES ('',$artist_id, $album_id)";
    my $sth = $dbh->prepare ($query);
    $sth->execute ();
  }
}
 
sub parse_filename ($)
# parse the filename in path, artists, title, extension
# strip off AUDIO_DIR first
{
  my ($path, $artists, $title, $extension);
  my $inputname = shift;
  
  # FIXME : add more filename checking here
  $inputname =~ /^$AUDIO_DIR(.+)\s+-\s+(.+)\.([^\.]+)$/;
  if (!$1 || !$2 || !$3)
  {
    logger ("E3", "could not parse file name into components");
    end_program ();
  }
  
  my $pathartists = $1;
  $title = $2;
  $extension = $3;

  if ($pathartists =~ /\//)
  {
    # it contains a path separator, so extract
    $pathartists =~ /^(.+)\/(.+)/;
    $path = $1;
    $artists = $2;
  }
  else
  {
    # no path, so it's the current dir
    # FIXME: maybe we should put the current path here ?
    $path = get_pwd ();
    $artists = $pathartists;
  }
      
  chomp ($extension);
  return ($path, $artists, $title, $extension);
}

sub get_artists ($)
# check if the given artistlist string contains more than one artist
# (separated with &) and return a list of separate artists
{
  my $artists = shift;
  my $artist;

  my @artistlist = split " & ", $artists;

  foreach $artist (@artistlist)
  {
    #print "DEBUG: $artist\n";
  }
  return @artistlist;
}

sub get_pwd ()
# return the current directory
{
 my $pwd = `pwd`;
 chomp $pwd;
 return $pwd;
}


sub usage ()
# print usage information
{
  print "$PROGRAM $version, (C) thomas\@apestaart.org\n";
  print "\n";
  print <<END;
-h     help
END
}

sub end_program ($)
# log exit and exit
{
  my $dbh = shift;
  if ($dbh) { $dbh->disconnect (); }
  logger ("S3", "stopped.");
  exit;
}

# main program

logger ("S3", "started.");

# check arguments
check_arguments (\%opt);

# check helper programs
#check_programs ();

# connect to database
my $dbh = connect_database ();

print "dadalbumimporting ".$opt{"f"}." ...\n";
# get filename components;
my ($path, $artists, $title, $extension) = parse_filename ($opt{"f"});
logger ("D4", "path $path, artists $artists, title $title, extension $extension");
my @artist_list = get_artists ($artists);
#print "DEBUG: main: artist list : (@artist_list)\n";
my @artist_id_list = db_get_artist_id_list ($dbh, @artist_list);

# add the album if it's not in the database
my $album_id = db_get_album_id ($dbh, $title, @artist_id_list); 

# insert in the album-artist-cross table data
db_album_artist_cross ($dbh, $album_id, @artist_id_list);

# read the file and process each of the audiofiles in it 
my $trackname;
my $tracknum = 0;
open ALBUM, $opt{"f"};
my @album = <ALBUM>;
close ALBUM;

foreach $trackname (@album)
{
  chomp $trackname;
  ++$tracknum;
  
  my $audiofile_id = db_get_audiofile_id ($dbh, "$AUDIO_DIR$path/$trackname");
  my $audio_id = db_get_audio_id ($dbh, $audiofile_id);
  db_audio_add_to_album ($dbh, $audio_id, $album_id, $tracknum);

  logger ("I3", "track $trackname added to album $title");
}
end_program ($dbh);
